<?php

namespace App\Contract;

use Symfony\Component\Uid\Uuid;

class OrderDto
{
    public function __construct(
        private Uuid $orderId,
        private Uuid $clientId,
        /** @var ProductDto[] */
        private array $products,
    ) {
    }

    public function getOrderId(): Uuid
    {
        return $this->orderId;
    }

    public function setOrderId(Uuid $orderId): void
    {
        $this->orderId = $orderId;
    }

    public function getClientId(): Uuid
    {
        return $this->clientId;
    }

    public function setClientId(Uuid $clientId): void
    {
        $this->clientId = $clientId;
    }

    public function getProducts(): array
    {
        return array_values($this->products);
    }

    public function setProducts(array $products): void
    {
        $this->products = array_combine(
            array_map(fn (ProductDto $product) => $product->getProductId(), $products),
            array_values($products)
        );
    }

    public function addProduct(ProductDto $product): void
    {
        $this->products[$product->getProductId()] = $product;
    }

    public function removeProduct(ProductDto $product): void
    {
        $this->products = array_filter($this->products, fn (ProductDto $other) => $other->getProductId() != $product->getProductId());
    }
}
