<?php

namespace App\Controller;

use App\Contract\ClientDto;
use App\Entity\Client;
use App\Exception\ClientAlreadyExistsException;
use App\Service\ClientService;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\HttpKernel\Attribute\MapRequestPayload;
use Symfony\Component\Routing\Attribute\Route;

#[Route(path: '/clients', name: 'clients_')]
#[AsController]
readonly class ClientController
{
    public function __construct(
        private ClientService $clientService,
    ) {
    }

    #[Route(name: 'create', methods: ['POST'])]
    public function create(#[MapRequestPayload] ClientDto $clientDto): Response
    {
        $client = new Client(
            clientId: $clientDto->getClientId(),
            name: $clientDto->getName(),
            initialBalance: $clientDto->getBalance(),
        );

        try {
            $this->clientService->createNewClient($client);
        } catch (ClientAlreadyExistsException) {
            return new Response(status: Response::HTTP_CONFLICT);
        }

        return new Response(status: Response::HTTP_CREATED);
    }
}
